<?php

namespace App\Jobs;

use Illuminate\Bus\Queueable;
use Illuminate\Queue\InteractsWithQueue;
use Illuminate\Contracts\Queue\ShouldQueue;
use App\Models\RechargeApi;
use App\Models\IterableApi;
use Log;

class IterableSubscriptionCreateEvent implements ShouldQueue
{
    use InteractsWithQueue, Queueable;

    private $iterableObj;
    private $rechargeObj;
    private $customerID;
    private $properties;
    private $requestData;

    /**
     * Create a new job instance.
     *
     * @return void
     */
    public function __construct($requestData) {

        $this->requestData = $requestData;
        $this->customerID  = $this->requestData->subscription->customer_id;
        $this->properties = $this->requestData->subscription->properties;
    }

    public function handle() {

        $this->iterableObj = new IterableApi();
        $this->rechargeObj = new RechargeApi();
        
        $customerDetails = $this->rechargeObj->getRechargeCustomerDetailsByID($this->customerID);

        if (isset($customerDetails->error) || isset($customerDetails->errors) || empty($customerDetails->customer->email) ||
            empty($customerDetails)) {
            $status = "failed";
            $error = "Unknown error";
            if (isset($customerDetails->error)) {
                $error = json_encode($customerDetails->error);
            } else if (isset($customerDetails->errors)) {
                $error = json_encode($customerDetails->errors);
            } else {
                $error = "Not found any customer details by customerId";
            }
            Log::channel('iterableWebhookLog')->info("IterableSubscriptionCreateEvent: Failed to get the Recharge user details for 
            CustomerID (".$this->customerID."), Error: " . $error);
            exit();
        }
        $nextChargeDate = '';
        $subscriptionLineItemsArray = [];
        $subscriptionLineItems = '';
        if (!empty($this->properties)) {
            $nextChargeDate = !empty($this->requestData->subscription->next_charge_scheduled_at) ?
            trim($this->requestData->subscription->next_charge_scheduled_at) : '';
            foreach ($this->properties as $chargeValue) {
                $propertyName = strtolower(trim($chargeValue->name));
                if (strpos($propertyName, 'vitamin') !== false) {
                    $propertyValue = trim($chargeValue->value);
                    if(strpos($propertyValue, '|') !== false) {
                        $explodePropertyValue = explode('|', $propertyValue);
                        if(count($explodePropertyValue) == 5) {
                            $subscriptionLineItemsArray[] = trim($explodePropertyValue[0]);
                        }
                    }
                }

            }
        }
        if (isset($subscriptionLineItemsArray) || !empty($subscriptionLineItemsArray)) {
            $subscriptionLineItems = implode(' | ', $subscriptionLineItemsArray);
        }
        if (!empty($nextChargeDate)) {
            $nextChargeDate = strtotime($nextChargeDate);
            $nextChargeDate = date("Y-m-d H:i:s", $nextChargeDate);
        }

        $customerSubscriptionDetails = $this->rechargeObj->getAllActiveSubscriptionsByCustomerID($this->customerID);

        if (isset($customerSubscriptionDetails->error) || isset($customerSubscriptionDetails->errors) ||
            empty($customerSubscriptionDetails) || !isset($customerSubscriptionDetails->subscriptions)) {
            $status = "failed";
            $error = "Unknown error";
            if (isset($customerSubscriptionDetails->error)) {
                $error = json_encode($customerSubscriptionDetails->error);
            } else if (isset($customerSubscriptionDetails->errors)) {
                $error = json_encode($customerSubscriptionDetails->errors);
            } else {
                $error = "No subscriptions were found in the recharge API response";
            }
            Log::channel('iterableWebhookLog')->info("IterableSubscriptionCreateEvent: Failed to get the Recharge user 
            subscription details by CustomerID (".$this->customerID."), Error: " . $error);
            exit();
        }
        if (count($customerSubscriptionDetails->subscriptions) > 1) {
            $nextChargeDate = NULL;
            $subscriptionLineItems = "Multiple Subscriptions";
        }
        $utm_campaignID = '';
        if (!empty($this->properties)) {
            foreach ($this->properties as $propertyValue) {
                if (strtolower(trim($propertyValue->name)) == 'utm') {
                    $utm_campaignID = $propertyValue->value;
                }
            }
        }
        $eventRequest = array(
            "email" => strtolower($customerDetails->customer->email), 
            "eventName" => "Subscription created",
            "dataFields" => array(
                "userName" => $customerDetails->customer->first_name.' '.$customerDetails->customer->last_name,
                "utm_campaign" => $utm_campaignID
            )
        );
        $userIterableUpdateRequest = array(
            "email" => $customerDetails->customer->email,
            "dataFields" => array(
                "rechargeHash" => $customerDetails->customer->hash,
                "nextChargeDate" => $nextChargeDate,
                "subscriptionItems" => $subscriptionLineItems
            )
        );

        $userIterableUpdateResponse = $this->iterableObj->updateUserDataFields(json_encode($userIterableUpdateRequest));

        if (strtolower(trim($userIterableUpdateResponse->code)) == 'success') {
            Log::channel('iterableWebhookLog')->info("IterableSubscriptionCreateEvent: User rechargeHash and subscription properties - Successfully
            updated the user recharge Hash and properties in Iterable for Email (".strtolower($customerDetails->customer->email).")");
        } else {
            Log::channel('iterableWebhookLog')->info("IterableSubscriptionCreateEvent: User rechargeHash and subscription properties - Failed to update
            the user recharge Hash and properties in Iterable for Email (".strtolower($customerDetails->customer->email).")");
        }

        $eventResponse = $this->iterableObj->createIterableEvent(json_encode($eventRequest));

        if (isset($eventResponse->code) || trim(strtolower($eventResponse->code)) == 'success') {
            Log::channel('iterableWebhookLog')->info("IterableSubscriptionCreateEvent: Created subscription iterable event - 
            Successfully created event for Email (".strtolower($customerDetails->customer->email).")");
        } else {
            Log::channel('iterableWebhookLog')->info("IterableSubscriptionCreateEvent: Created subscription iterable event - 
            Failed to create event for Email (".strtolower($customerDetails->customer->email).")");
        }
    }
}
